#ifndef PICOLED_GFX_2D_H
#define PICOLED_GFX_2D_H

#include "picoled.h"

namespace picoled::gfx
{

class vec2
{
public:
  float x;
  float y;

  vec2 operator+(const vec2& other) const { return { x + other.x, y + other.y }; }
  vec2 operator-(const vec2& other) const { return { x - other.x, y - other.y }; }
  vec2 operator*(const vec2& other) const { return { x * other.x, y * other.y }; }
  vec2 operator/(const vec2& other) const { return { x / other.x, y / other.y }; }

  float operator%(const vec2& other) const { return x * other.x + y * other.y; }
  float operator^(const vec2& other) const { return x * other.y - y * other.x; }
};

class triangle
{
public:
  vec2 a;
  vec2 b;
  vec2 c;

  triangle fix_winding() const;
  bool inside(const vec2& p) const;

  vec2 min() const;
  vec2 max() const;
};

class gfx2d
{
public:
  explicit gfx2d(oled& screen) :
    screen(&screen)
  {}

  gfx2d(const gfx2d&) = delete;
  gfx2d(gfx2d&&) = delete;

  gfx2d& operator=(const gfx2d&) = delete;
  gfx2d& operator=(gfx2d&&) = delete;

  void draw_triangle(triangle tri, pixel p = 0xFF, bool ignore_winding = true);

private:
  oled* screen;
};

}

#endif//PICOLED_GFX_2D_H
