#include "picoled/gfx/2D.h"

#include <cmath>

namespace picoled::gfx
{

triangle triangle::fix_winding() const
{
  if (((b - a) ^ (c - a)) < 0)
  {
    return { a, c, b };
  }
  else
  {
    return { a, b, c };
  }
}

bool triangle::inside(const vec2& p) const
{
  const float abp = (b - a) ^ (p - a);
  const float bcp = (c - b) ^ (p - b);
  const float cap = (a - c) ^ (p - c);

  return abp >= 0 && bcp >= 0 && cap >= 0;
}

vec2 triangle::min() const
{
  return
  {
    fminf(fminf(a.x, b.x), c.x),
    fminf(fminf(a.y, b.y), c.y)
  };
}

vec2 triangle::max() const
{
  return
  {
    fmaxf(fmaxf(a.x, b.x), c.x),
    fmaxf(fmaxf(a.y, b.y), c.y)
  };
}

void gfx2d::draw_triangle(triangle tri, pixel p, bool ignore_winding)
{
  if (ignore_winding) { tri = tri.fix_winding(); }

  const vec2 min = tri.min();
  const vec2 max = tri.max();

  for (float i = min.x; i <= max.x; i += 0.75f)
  {
    for (float j = min.y; j <= max.y; j += 0.75f)
    {
      if (tri.inside({ i, j })) { screen->write(i, j, p); }
    }
  }
}

}
